/**
 * AutoRegister
 *
 * Auto register angular modules that are loaded so all you have to do is
 * import them as needed and don't need to manually add angular dependencies
 *
 * tl;dr you don't ever have to do angular.module 'app', [...stuff], instead
 * just
 *
 * 		import 'what-you-want';
 *
 * and any angular modules imported are automatically loaded for you as
 * a dependency of your application
 *
 * @todo move to node module / bower component
 */
var AutoRegister = (function () {
    function AutoRegister(appModuleName) {
        if (appModuleName === void 0) { appModuleName = 'app'; }
        this.modules = [];
        this.appModuleName = appModuleName;
        this.registerModule();
        this.stubModuleFunction();
    }
    /**
     * Register our main application module that everything gets bound to
     */
    AutoRegister.prototype.registerModule = function () {
        angular.module(this.appModuleName, this.modules);
    };
    /**
     * Stub angular.module to listen to registered modules
     * and automatically register them for the app module
     */
    AutoRegister.prototype.stubModuleFunction = function () {
        var _this = this;
        var originalAngularModuleFunction = angular.module;
        angular.module = function (moduleName, dependencies) {
            var otherArgs = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                otherArgs[_i - 2] = arguments[_i];
            }
            // If the module being registered is not yet in our app module
            // dependencies, add it
            if (dependencies && _this.modules.indexOf(moduleName) === -1) {
                _this.modules.push(moduleName);
            }
            return originalAngularModuleFunction.call.apply(originalAngularModuleFunction, [angular, moduleName, dependencies].concat(otherArgs));
        };
    };
    return AutoRegister;
})();
exports.AutoRegister = AutoRegister;
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = new AutoRegister('app');
