var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var config_1 = require('../decorators/config');
var inject_1 = require('../decorators/inject');
var JsonEditorSettings = (function () {
    function JsonEditorSettings() {
        this.createTheme();
        this.createIconLib();
        this.setProviderConfigs();
        this.setAcePluginConfigs();
        this.addUploadHandler();
        this.addUploadEditor();
        this.addDateType();
    }
    JsonEditorSettings.prototype.addDateType = function () {
        JSONEditor.defaults.editors.date = JSONEditor.AbstractEditor.extend({});
    };
    // TODO: move me and make configurable
    JsonEditorSettings.prototype.addUploadHandler = function () {
        JSONEditor.defaults.options.upload = function (type, file, cbs) {
            if (type === 'root.upload_fail') {
                cbs.failure('Upload failed');
            }
            else {
                // TODO: better way of getting $http service
                // maybe move this to run block instead of config
                var injector = angular.element(document).injector();
                var $http = injector.get('$http');
                var $ionicLoading = injector.get('$ionicLoading');
                var formData = new FormData();
                // TODO: real UUID
                var randomInt = Math.random().toString().split('.')[1];
                var fileExtension = _.last(file.name.split('.'));
                var now = Date.now();
                var filename = now + "-" + randomInt + "." + fileExtension;
                formData.append('file', file, filename);
                var bucketName = 'shopstyle-cms-qa-sandbox';
                $ionicLoading.show();
                $http
                    .post("/api/v1/fileUploads/" + bucketName + "/upload", formData, {
                    transformRequest: angular.identity,
                    headers: { 'Content-Type': undefined }
                })
                    .success(function (data) {
                    $ionicLoading.hide();
                    cbs.success("http://s3.amazonaws.com/" + bucketName + "/" + filename);
                });
            }
        };
    };
    JsonEditorSettings.prototype.addUploadEditor = function () {
        JSONEditor.defaults.editors.upload = JSONEditor.defaults.editors.upload.extend({
            build: function () {
                var _this = this;
                this._super();
                this.uploader.setAttribute('accept', 'image/*');
                var clearButton = this.clearButton = document.createElement('a');
                clearButton.className = 'button button-small button-icon ion-close-round clear-input-button';
                clearButton.addEventListener('click', function (event) {
                    _this.setValue('');
                    if (_this.parent) {
                        _this.parent.onChildEditorChange(_this);
                    }
                    else {
                        _this.jsoneditor.onChange();
                    }
                });
                this.control.appendChild(clearButton);
                this.uploader.addEventListener('change', function (event) {
                    var file = _this.uploader.files[0];
                    _this.jsoneditor.options.upload(_this.path, file, {
                        success: function (url) {
                            _this.setValue(url);
                            if (_this.parent) {
                                _this.parent.onChildEditorChange(_this);
                            }
                            else {
                                _this.jsoneditor.onChange();
                            }
                        },
                        failure: function (error) {
                            _this.theme.addInputError(_this.uploader, error);
                        }
                    });
                });
            },
            setValue: function (value) {
                this._super(value);
                this.control.style.backgroundImage = "url('" + value + "')";
                if (value) {
                    this.clearButton.style.display = 'inline-block';
                }
                else {
                    this.clearButton.style.display = 'none';
                }
            }
        });
    };
    JsonEditorSettings.prototype.setAcePluginConfigs = function () {
        JSONEditor.plugins.ace.theme = 'tomorrow';
    };
    JsonEditorSettings.prototype.setProviderConfigs = function () {
        this.JSONEditorProvider.configure({
            defaults: {
                options: {
                    iconlib: 'ionic',
                    theme: 'ionic',
                    ajax: true,
                    disable_edit_json: true,
                    no_additional_properties: true,
                    disable_properties: true,
                    required_by_default: true
                }
            }
        });
    };
    JsonEditorSettings.prototype.createIconLib = function () {
        JSONEditor.defaults.iconlibs.ionic = JSONEditor.AbstractIconLib.extend({
            mapping: {
                collapse: 'ion-chevron-down',
                expand: 'ion-chevron-right',
                delete: 'ion-close-round',
                edit: 'ion-edit',
                add: 'ion-plus-round',
                cancel: 'ion-android-cancel',
                save: 'ion-ios-download',
                moveup: 'ion-arrow-up-c',
                movedown: 'ion-arrow-down-c'
            },
            icon_prefix: 'icon '
        });
    };
    JsonEditorSettings.prototype.createTheme = function () {
        JSONEditor.defaults.themes.ionic = JSONEditor.AbstractTheme.extend({
            getSelectInput: function (options) {
                var el = this._super(options);
                return el;
            },
            setGridColumnSize: function (el, size) {
                el.className = 'col-md-' + size;
            },
            afterInputReady: function (input) {
                if (input.controlgrfoup) {
                    return;
                }
                input.controlgroup = this.closest(input, '.item');
                if (this.closest(input, '.compact')) {
                    input.controlgroup.style.marginBottom = 0;
                }
            },
            getTextareaInput: function () {
                var el = document.createElement('textarea');
                return el;
            },
            getRangeInput: function (min, max, step) {
                // TODO: use better slider
                return this._super(min, max, step);
            },
            getFormInputField: function (type) {
                var el = this._super(type);
                el.placeholder = el.placeholder || el.type;
                return el;
            },
            getFormInputLabel: function (text) {
                text = _.startCase(text);
                var el = document.createElement('label');
                el.appendChild(document.createTextNode(text));
                return el;
            },
            getFormControl: function (label, input, description) {
                var group = document.createElement('div');
                label.innerText = _.startCase(label.innerText);
                if (label && input.type === 'checkbox') {
                    group.className += ' checkbox';
                    label.appendChild(input);
                    group.appendChild(label);
                }
                else {
                    group.className += ' item item-input';
                    var inputTagType = input.tagName.toLowerCase();
                    if (inputTagType !== 'input') {
                        // Select, textarea
                        group.className += ' item-' + inputTagType;
                    }
                    if (label) {
                        label.className += ' input-label';
                        group.appendChild(label);
                    }
                    group.appendChild(input);
                }
                if (description) {
                    group.appendChild(description);
                }
                return group;
            },
            getIndentedPanel: function () {
                var el = document.createElement('div');
                el.className = 'indented';
                return el;
            },
            // TODO: modify to be tooltip
            getFormInputDescription: function (text) {
                var el = document.createElement('p');
                el.className = 'help-block';
                el.innerHTML = text;
                return el;
            },
            getHeaderButtonHolder: function () {
                var el = this.getButtonHolder();
                return el;
            },
            getButtonHolder: function () {
                var el = document.createElement('span');
                // el.className = 'button-bar';
                return el;
            },
            getButton: function (text, icon, title) {
                var el = this._super(text, icon, title);
                el.className += 'button button-small icon-left button-clear';
                return el;
            },
            getTable: function () {
                var el = document.createElement('table');
                el.className = 'table table-bordered';
                el.style.width = 'auto';
                el.style.maxWidth = 'none';
                return el;
            },
            addInputError: function (input, text) {
                if (!input.controlgroup) {
                    return;
                }
                input.controlgroup.className += ' has-error';
                if (!input.errmsg) {
                    input.errmsg = document.createElement('p');
                    input.errmsg.className = 'help-block errormsg';
                    input.controlgroup.appendChild(input.errmsg);
                }
                else {
                    input.errmsg.style.display = '';
                }
                input.errmsg.textContent = text;
            },
            removeInputError: function (input) {
                if (!input.errmsg) {
                    return;
                }
                input.errmsg.style.display = 'none';
                input.controlgroup.className = input.controlgroup.className.replace(/\s?has-error/g, '');
            },
            // getTabHolder: function() {
            //   const el = document.createElement('div');
            //
            //   el.innerHTML = `
            //     <div class='tabs list-group col-md-2'>
            //     </div>
            //     <div class='col-md-10'
            //     ></div>
            //   `;
            //
            //   el.className = 'rows';
            //   return el;
            // },
            getTab: function (text) {
                var el = document.createElement('a');
                el.className = 'list-group-item';
                el.setAttribute('href', '#');
                el.appendChild(text);
                return el;
            },
            markTabActive: function (tab) {
                tab.className += ' active';
            },
            markTabInactive: function (tab) {
                tab.className = tab.className.replace(/\s?active/g, '');
            },
            getProgressBar: function () {
                var min = 0, max = 100, start = 0;
                var container = document.createElement('div');
                container.className = 'progress';
                var bar = document.createElement('div');
                bar.className = 'progress-bar';
                bar.setAttribute('role', 'progressbar');
                bar.setAttribute('aria-valuenow', start.toString());
                bar.setAttribute('aria-valuemin', min.toString());
                bar.setAttribute('aria-valuenax', max.toString());
                bar.innerHTML = start + '%';
                container.appendChild(bar);
                return container;
            },
            getGridContainer: function () {
                var el = document.createElement('div');
                el.className = 'list';
                return el;
            },
            getGridRow: function () {
                var el = document.createElement('div');
                // el.className = 'list';
                return el;
            },
            updateProgressBar: function (progressBar, progress) {
                if (!progressBar) {
                    return;
                }
                var bar = progressBar.firstChild;
                var percentage = progress + '%';
                bar.setAttribute('aria-valuenow', progress);
                bar.style.width = percentage;
                bar.innerHTML = percentage;
            },
            updateProgressBarUnknown: function (progressBar) {
                if (!progressBar) {
                    return;
                }
                var bar = progressBar.firstChild;
                progressBar.className = 'progress progress-striped active';
                bar.removeAttribute('aria-valuenow');
                bar.style.width = '100%';
                bar.innerHTML = '';
            }
        });
    };
    __decorate([
        inject_1.inject, 
        __metadata('design:type', Object)
    ], JsonEditorSettings.prototype, "JSONEditorProvider", void 0);
    JsonEditorSettings = __decorate([
        config_1.Config, 
        __metadata('design:paramtypes', [])
    ], JsonEditorSettings);
    return JsonEditorSettings;
})();
exports.JsonEditorSettings = JsonEditorSettings;
