// TODO: separate test dependencies
require('@host');
require('angular-mocks');
require('mocha');
var chai_1 = require('chai');
var _ = require('lodash');
describe('objectUtils', function () {
    beforeEach(angular.mock.module('app'));
    describe('clear', function () {
        it('should return the original object/array reference', inject(function (objectUtils) {
            var object = { foo: 'bar' };
            var returned = objectUtils.clear(object);
            chai_1.expect(returned).to.equal(object);
        }));
        it('should clear all object keys', inject(function (objectUtils) {
            var object = { foo: 'bar' };
            objectUtils.clear(object);
            // the object should have no keys
            chai_1.expect(_.size(object)).to.equal(0);
        }));
        it('should clear all object keys', inject(function (objectUtils) {
            var array = [1, 2, 3];
            objectUtils.clear(array);
            // the object should have no items
            chai_1.expect(_.size(array)).to.equal(0);
        }));
    });
    describe('remove', function () {
        it('should return the original array reference', inject(function (objectUtils) {
            var array = [1, 2, 3];
            var returned = objectUtils.clear(array);
            chai_1.expect(returned).to.equal(array);
        }));
        it('should remove an item from an array', inject(function (objectUtils) {
            var array = [1, 2, 3, 2];
            var returned = objectUtils.remove(array, 2);
            // the original array should not contain the removed value anymore
            chai_1.expect(returned).to.not.contain(2);
        }));
    });
    describe('replace', function () {
        it('should return the original object reference', inject(function (objectUtils) {
            var obj = {};
            var returned = objectUtils.replace(obj, { foo: 'bar' });
            chai_1.expect(returned).to.equal(obj);
        }));
        it('should replace all keys and values of one object to another', inject(function (objectUtils) {
            var obj1 = { foo: 'bar' };
            var obj2 = { bar: 'foo' };
            objectUtils.replace(obj1, obj2);
            chai_1.expect(obj1.foo).to.be.undefined;
            chai_1.expect(obj1.bar).to.equal('foo');
        }));
        it('shoudl replace all array items of one array to another', inject(function (objectUtils) {
            var arr1 = [1, 2, 3];
            var arr2 = [0, 1, 2];
            var returned = objectUtils.replace(arr1, arr2);
            chai_1.expect(returned).to.equal(arr1);
            for (var _i = 0; _i < arr2.length; _i++) {
                var item = arr2[_i];
                chai_1.expect(arr1[item]).to.equal(item);
            }
        }));
    });
});
