export declare class UrlUtils {
    /**
     * Encode and add a param to a url
     *
     * @example
     * ```ts
     * urlUtils.addParam('foo.com', 'foo', 'b a r');
     * // returns 'foo.com?foo=b%20a%20r'
     *
     * utils.addParam('foo.com?already=hasParams', 'foo', 'b a r');
     * // returns 'foo.com?already=hasParams&foo=b%20a%20r'
     * ```
     *
     * @param  {string} url   Url to add param to
     * @param  {string} name  Param key
     * @param  {string} value Param value
     * @return {string}       Newly const ructed url
     */
    addParam(url: string, name: string, value: string): string;
    /**
     * Encode a URI compoent angular style
     */
    encodeURIComponent(str: string): string;
    /**
     * Parse query params from a URL string
     */
    getQueryParams(url: string): {};
    /**
     * Encodes and adds a map of param names/values sorting them
     * alphabetically for consistent cache keys
     *
     * @example
     * ```ts
     * urlUtils.addParams('foo.com', {
     * 	 foo: 'bar',
     * 	 list: ['a', 'b', 'c']
     * });
     * // returns 'foo.com?foo=bar&list=a&list=b&list=c'
     * ```
     *
     * @param  {string} url      Start URL
     * @param  {any}    paramMap Map of params to add
     * @return {string}          New URL
     */
    addParams(url: string, paramMap: any): string;
    /**
     * Generates the absolute path for a url when
     * it is within a cordova app. Necessary because
     * lazy loading/webpack messes with how cordova usually
     * does this automatically.
    */
    getAbsolutePathForCordova(url?: string): string;
    parseUrl(url: string): {
        href: string;
        protocol: string;
        host: string;
        port: string;
        path: string;
        search: string;
        query: any;
    };
}
