var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var constant_1 = require('../../decorators/constant');
var UrlUtils = (function () {
    function UrlUtils() {
    }
    /**
     * Encode and add a param to a url
     *
     * @example
     * ```ts
     * urlUtils.addParam('foo.com', 'foo', 'b a r');
     * // returns 'foo.com?foo=b%20a%20r'
     *
     * utils.addParam('foo.com?already=hasParams', 'foo', 'b a r');
     * // returns 'foo.com?already=hasParams&foo=b%20a%20r'
     * ```
     *
     * @param  {string} url   Url to add param to
     * @param  {string} name  Param key
     * @param  {string} value Param value
     * @return {string}       Newly const ructed url
     */
    UrlUtils.prototype.addParam = function (url, name, value) {
        value = this.encodeURIComponent(value);
        if (url.indexOf('?') < 0) {
            url += '?';
        }
        else {
            url += '&';
        }
        url += name + '=' + value;
        return url;
    };
    /**
     * Encode a URI compoent angular style
     */
    UrlUtils.prototype.encodeURIComponent = function (str) {
        str = decodeURIComponent(str);
        return encodeURIComponent(str)
            .replace(/%40/gi, '@')
            .replace(/%3A/gi, ':')
            .replace(/%3B/gi, ';')
            .replace(/%24/g, '$')
            .replace(/%2C/gi, ',')
            .replace(/%20/g, '+');
    };
    /**
     * Parse query params from a URL string
     */
    UrlUtils.prototype.getQueryParams = function (url) {
        var queryString = url.split('?')[1] || url;
        var queryParams = {};
        queryString
            .split('&')
            .forEach(function (keyVal) {
            var _a = keyVal.split('='), key = _a[0], value = _a[1];
            var currentValue = queryParams[key];
            if (currentValue) {
                if (!_.isArray(currentValue)) {
                    queryParams[key] = [currentValue];
                }
                queryParams[key].push(value);
            }
            else {
                queryParams[key] = value;
            }
        });
        return queryParams;
    };
    /**
     * Encodes and adds a map of param names/values sorting them
     * alphabetically for consistent cache keys
     *
     * @example
     * ```ts
     * urlUtils.addParams('foo.com', {
     * 	 foo: 'bar',
     * 	 list: ['a', 'b', 'c']
     * });
     * // returns 'foo.com?foo=bar&list=a&list=b&list=c'
     * ```
     *
     * @param  {string} url      Start URL
     * @param  {any}    paramMap Map of params to add
     * @return {string}          New URL
     */
    UrlUtils.prototype.addParams = function (url, paramMap) {
        var _this = this;
        Object
            .keys(paramMap)
            .sort()
            .forEach(function (name) {
            var value = paramMap[name];
            if (_.isArray(value)) {
                for (var _i = 0; _i < value.length; _i++) {
                    var item = value[_i];
                    url = _this.addParam(url, name, item);
                }
            }
            else {
                url = _this.addParam(url, name, value);
            }
        });
        return url;
    };
    /**
     * Generates the absolute path for a url when
     * it is within a cordova app. Necessary because
     * lazy loading/webpack messes with how cordova usually
     * does this automatically.
    */
    UrlUtils.prototype.getAbsolutePathForCordova = function (url) {
        if (url === void 0) { url = ''; }
        var path = window.location.pathname.split('/');
        path.pop();
        return path.join('/') + '/' + url;
    };
    UrlUtils.prototype.parseUrl = function (url) {
        // returns an anchor tag from which you can access
        // protocol, hostname, port, pathname, search, hash
        if (!url) {
            return;
        }
        var parser = document.createElement('a');
        parser.href = url;
        var parsedUrl = {
            href: url,
            protocol: parser.protocol,
            host: parser.hostname,
            port: parser.port,
            path: (function () {
                if (/^\//.test(parser.pathname)) {
                    return parser.pathname;
                }
                else {
                    // the parsed pathname on IE doesn't contain a preceding /
                    // we need to manually add it or our generated hrefs will be relative
                    return "/" + parser.pathname;
                }
            })(),
            search: parser.search,
            query: (function () {
                var query = {};
                var queryParams = parser.search.substr(1).split('&');
                queryParams.forEach(function (param) {
                    var pair = param.split('=');
                    query[pair[0]] = pair[1];
                });
                return query;
            })(),
        };
        return parsedUrl;
    };
    UrlUtils = __decorate([
        constant_1.Constant('urlUtils', true), 
        __metadata('design:paramtypes', [])
    ], UrlUtils);
    return UrlUtils;
})();
exports.UrlUtils = UrlUtils;
