/**
 * @decorator Constant
 * @description create and set angular constants
 * @param {string} name           name of constant to set
 * @param {string} createInstance create an instance of this class and set
 * that as the config value
 * @type {Function}
 * @example
 * ```ts
 * @Constant('Router')
 * export class Router {
 *   // ...
 * }
 * ```
 */
exports.Constant = function (name, createInstance) {
    if (createInstance === void 0) { createInstance = false; }
    var decorator = function (target) {
        if (createInstance) {
            target = new target();
        }
        angular
            .module('app')
            .constant(name, target);
    };
    return decorator;
};
