/**
 * @decorator inject
 *
 * @type {PropertyDecorator}
 *
 * @todo Allow this to take options like @inject('name') so can alias?
 */
exports.inject = function (target, name) {
    var constructor = target.constructor;
    constructor.$inject = (constructor.$inject || []).slice();
    constructor.$inject.push(name.toString());
};
/**
 * Make a class angular injectable
 *
 * @param {any}     Target
 * @param {boolean} createInstance  Should an instance of this class be created
 *
 * @todo: find a better place for this
 * @todo: make a class or decorator that is subclassed or used on decorator?
 */
exports.makeInjectable = function (Target, createInstance, bindContext) {
    if (createInstance === void 0) { createInstance = false; }
    if (bindContext === void 0) { bindContext = false; }
    var injectList = Target.$inject || [];
    // Take in services injected by angular and attach
    // them to the class instance
    var init = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        injectList.forEach(function (dependencyName, index) {
            Target.prototype[dependencyName] = args[index];
        });
        if (createInstance) {
            return Target;
        }
        else {
            var instance = new Target();
            // In some cases class instace methods might be executed
            // without their instance context - like in the case of angular
            // interceptor methods, so we can optionally override
            // context if needed
            if (bindContext) {
                for (var key in instance) {
                    var value = instance[key];
                    if (typeof value === 'function') {
                        instance[key] = value.bind(instance);
                    }
                }
            }
            return instance;
        }
    };
    // Construct the angular annotation notation for this class
    // e.g. ['myService', (myService) => {}]
    var conf = injectList.slice();
    conf.push(init);
    return conf;
};
