var inject_1 = require('./inject');
/**
 * @decorator Config
 *
 * @type {ClassDecorator}
 *
 * @example
 *
 * ```ts
 * @Interceptor
 * export class Routes {
 * 	 @inject
 * 	 private someDependency: SomeDependency;
 *
 * 		// Constructor takes no args for config
 * 		// classes
 * 		request(config: angular.IRequestConfig) {
 * 			if (this.someDependency.foo()) {
 * 				config.params.bar = 'baz';
 * 			}
 * 		}
 * }
 * ```
 */
exports.Interceptor = function (Target) {
    var target = inject_1.makeInjectable(Target, false, true);
    // Construct the module
    angular
        .module('app')
        .config(['$httpProvider', function ($httpProvider) {
            $httpProvider.interceptors.push(target);
        }]);
};
