var inject_1 = require('./inject');
var serviceCache = {};
/**
 * @decorator Service
 *
 * @description create angular services
 *
 * @param {string} name
 * name of your service
 *
 * @example
 * ```ts
 * @Service('abTests')
 * export class AbTests {
 *   @inject str: Str;
 *
 * 	 onstructor() {
 * 	   // do something...
 * 	   str.dasherize('foobar');
 * 	 }
 * }
 * ```
 */
exports.Service = function (name) {
    var decorator = function (Target) {
        var target = inject_1.makeInjectable(Target);
        // TODO: do with constants, etc
        if (serviceCache[name]) {
            var injector = angular.element(document).injector();
            var invoked = injector.invoke(target);
            var original = injector.get(name);
            for (var key in original) {
                delete original[key];
            }
            // TODO: does not handle deletions - maybe wipe out the original
            // object first
            for (var key in invoked) {
                original[key] = invoked[key];
            }
        }
        else {
            // Construct the module
            angular
                .module('app')
                .service(name, target);
            serviceCache[name] = target;
        }
    };
    return decorator;
};
