/**
 * @interface IState
 * @extends angular.ui.IState
 * @description
 * Extends angular.ui.IState to support a comopnent property
 * that takes a component name and turns it into a template
 */
export interface IState extends angular.ui.IState {
    component?: string;
    /**
     * If there is a component for this state, should it be
     * lazy loaed?
     *
     * @type {Boolean}
     * @default false
     */
    lazy?: Boolean;
    resolve?: any;
    attachToView?: string;
    wrapTemplate?: boolean;
    /**
     * Source module to find component
     * @example
     *   module: '@ssjs/shopstyle-components'
     */
    module?: string;
    /**
     * Path to component
     */
    path?: string;
    hideNavBar?: boolean;
}
export interface IRouterDefaults {
    /**
     * Default route to redirect to if no others match
     * @type {string}
     */
    defaultRoute: string;
    /**
     * List of states
     * @type {IState[]}
     * ```
     */
    routes: IState[];
    /**
     * String to prepend to all component state names. E.g.
     * with the default 'app' a component by the name 'home'
     * wil lhave the state name 'app.home'
     * @todo way to configure these as a provider
     * @default 'app'
     * @type {String}
     */
    componentStatePrefix: string;
    /**
     * Name of main ui-view. This is what is used
     * for the view name all components nest under.
     * E.g. for 'menuContent' component routes get const ructed as
     *
     * @type {String}
     * @default 'menuContent';
     */
    mainViewName: string;
}
export declare class Router implements IRouterDefaults {
    $stateProvider: angular.ui.IStateProvider;
    $urlRouterProvider: angular.ui.IUrlRouterProvider;
    componentStatePrefix: string;
    mainViewName: string;
    defaultRoute: string;
    routes: IState[];
    static noop(): void;
    constructor();
    /**
     * Set router defaults via IRouterDefaults interface
     *
     * @param {IRouterDefaults} options   Router defaults to set
     * @param {boolean}         defaults  Only set if not already set
     */
    set(options: IRouterDefaults, defaults?: boolean): void;
    $get(): Router;
    /**
     * Takes a state object and const ructs and applies the route
     * for it via $stateProvider.state()
     *
     * @param  {IState}    state  State object to apply
     * @return {ui.IState}        Converted state object (to angular.ui.IState format)
     */
    applyRoute(state: IState): angular.ui.IState;
    /**
     * Converts an IState to a angular.ui.IState
     *
     * @description
     * Note that any property can be overwritten, so you can
     * supply custom urls, views, component name, etc and the
     * 'component' property will fill in any missing fields
     *
     * @param  {IState} state  Default state object to conert
     * @return {ui.IState}     Converted state object
     *
     * @example
     * Converts
     * ```ts
     * { component: 'home' }
     * ```
     * to
     * ```ts
     * {
     * 	name: 'app.home',
     * 	url: '/home',
     * 	views: {
     * 		menuContent: {
     * 			template: ```
     * 				<ion-view title="Home">
     * 					<ion-content>
     * 						<home></home>
     * 					</ion-content>
     * 				</ion-view>
     * 			```
     * 		}
     * 	}
     * }
     * ```
     */
    configureState(state: IState): angular.ui.IState;
    /**
     * Apply routes configured via this.routes
     * @param {IState[]} routes List
     */
    applyRoutes(routes?: IState[]): void;
}
