var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
require('@host');
var provider_1 = require('../decorators/provider');
var inject_1 = require('../decorators/inject');
var component_1 = require('../services/component');
var lazy_1 = require('../services/lazy');
var _ = require('lodash');
var Router = (function () {
    function Router() {
        /*
         * Property defaults
         */
        this.componentStatePrefix = 'app';
        this.mainViewName = 'menuContent';
        this.defaultRoute = '/';
        this.routes = [];
        // Initialize
        lazy_1.Lazy.noop();
    }
    Router.noop = function () {
        // do nothing
    };
    /**
     * Set router defaults via IRouterDefaults interface
     *
     * @param {IRouterDefaults} options   Router defaults to set
     * @param {boolean}         defaults  Only set if not already set
     */
    Router.prototype.set = function (options, defaults) {
        if (defaults === void 0) { defaults = false; }
        if (defaults) {
            _.defaults(options, this);
        }
        else {
            _.extend(options, this);
        }
    };
    // TODO: class Router extends SingletonProvider
    Router.prototype.$get = function () {
        return this;
    };
    /**
     * Takes a state object and const ructs and applies the route
     * for it via $stateProvider.state()
     *
     * @param  {IState}    state  State object to apply
     * @return {ui.IState}        Converted state object (to angular.ui.IState format)
     */
    Router.prototype.applyRoute = function (state) {
        var uiState = this.configureState(state);
        this.$stateProvider.state(state.name, uiState);
        return uiState;
    };
    /**
     * Converts an IState to a angular.ui.IState
     *
     * @description
     * Note that any property can be overwritten, so you can
     * supply custom urls, views, component name, etc and the
     * 'component' property will fill in any missing fields
     *
     * @param  {IState} state  Default state object to conert
     * @return {ui.IState}     Converted state object
     *
     * @example
     * Converts
     * ```ts
     * { component: 'home' }
     * ```
     * to
     * ```ts
     * {
     * 	name: 'app.home',
     * 	url: '/home',
     * 	views: {
     * 		menuContent: {
     * 			template: ```
     * 				<ion-view title="Home">
     * 					<ion-content>
     * 						<home></home>
     * 					</ion-content>
     * 				</ion-view>
     * 			```
     * 		}
     * 	}
     * }
     * ```
     */
    Router.prototype.configureState = function (state) {
        var _a = this, mainViewName = _a.mainViewName, componentStatePrefix = _a.componentStatePrefix;
        var component = state.component;
        var dashName = _.kebabCase(component);
        var camelName = _.camelCase(component);
        // Configure base state object, e.g.
        // {
        //  component: 'home'
        // }
        // to
        // {
        //  name: 'app.home'
        // }
        if (component) {
            // URL default
            if (_.isUndefined(state.url)) {
                state.url = "/" + dashName;
            }
            // name default
            if (!state.name) {
                state.name = componentStatePrefix + "." + camelName;
            }
            // wrap template default
            if (_.isUndefined(state.wrapTemplate)) {
                state.wrapTemplate = true;
            }
            if (!state.views) {
                var viewName = state.attachToView || mainViewName;
                state.views = (_b = {},
                    _b[viewName] = {
                        component: component
                    },
                    _b
                );
            }
            // Enable lazy loading
            if (state.lazy !== false && !state.abstract) {
                state.resolve = state.resolve || {};
                // TODO: ngannotate for this
                state.resolve.$lazy = ['lazy', function (lazy) {
                        return lazy.load(component);
                    }];
            }
        }
        // Configure views, e.g.
        // {
        //   component: 'main'
        // }
        // to
        // {
        //   template: '<ui-view title="Main">...</ui-view>'
        // }
        //
        for (var viewName in state.views) {
            var viewConfig = state.views[viewName];
            // If we see a component property turn it into
            // a template string instead via Component.viewTemplate
            if (viewConfig.component) {
                var componentInstance = new component_1.Component();
                if (state.wrapTemplate) {
                    viewConfig.template = componentInstance.viewTemplate(viewConfig.component);
                }
                else {
                    viewConfig.template = componentInstance.template(viewConfig.component);
                }
            }
        }
        return state;
        var _b;
    };
    /**
     * Apply routes configured via this.routes
     * @param {IState[]} routes List
     */
    Router.prototype.applyRoutes = function (routes) {
        if (routes === void 0) { routes = this.routes; }
        if (this.routes) {
            for (var _i = 0, _a = this.routes; _i < _a.length; _i++) {
                var state = _a[_i];
                this.applyRoute(state);
            }
        }
        // Set the default route
        if (this.defaultRoute) {
            this.$urlRouterProvider.otherwise(this.defaultRoute);
        }
    };
    __decorate([
        inject_1.inject, 
        __metadata('design:type', Object)
    ], Router.prototype, "$stateProvider", void 0);
    __decorate([
        inject_1.inject, 
        __metadata('design:type', Object)
    ], Router.prototype, "$urlRouterProvider", void 0);
    Router = __decorate([
        provider_1.Provider('router'), 
        __metadata('design:paramtypes', [])
    ], Router);
    return Router;
})();
exports.Router = Router;
