require('./dependencies');
var http = require('http');
var https = require('https');
var fs = require('fs');
var path = require('path');
var boot = require('loopback-boot');
var ShopStyleNodeServer = (function () {
    function ShopStyleNodeServer(config, loopback) {
        this.enableSsl = false;
        this.sslOptions = {};
        this.loopback = loopback;
        this.config = config;
        this.app = this.loopback();
        this.app.config = config;
        this.app.set('config', config);
        this.enableSsl = this.config.sslConfig && this.config.sslConfig.useSsl;
        if (this.enableSsl) {
            this.sslOptions.key = this.readSslFile(this.config.sslConfig.keyFile);
            this.sslOptions.cert = this.readSslFile(this.config.sslConfig.certFile);
        }
        this.boot();
        return this.app;
    }
    ShopStyleNodeServer.prototype.readSslFile = function (filePath) {
        return fs.readFileSync(path.join(__dirname, '../..', filePath), 'utf-8').toString();
    };
    ShopStyleNodeServer.prototype.boot = function () {
        var _this = this;
        // TODO: get from configs
        var bootPath = path.join(process.cwd(), 'server');
        // Bootstrap the application, configure models, datasources and middleware.
        // Sub-apps like REST API are mounted via boot scripts.
        boot(this.app, bootPath, function (err) {
            if (err) {
                throw err;
            }
            _this.start();
        });
    };
    ShopStyleNodeServer.prototype.start = function () {
        var _this = this;
        var server;
        if (this.enableSsl) {
            server = https.createServer(this.sslOptions, this.app);
        }
        else {
            server = http.createServer(this.app);
        }
        // start the web server
        server.listen(this.app.get('port'), function () {
            var baseUrl = (_this.enableSsl ? 'https://' : 'http://')
                + _this.app.get('host')
                + ':' + _this.app.get('port');
            _this.app.emit('started', baseUrl);
            console.log('Web server listening at: ' + baseUrl + '...');
        });
    };
    return ShopStyleNodeServer;
})();
exports.ShopStyleNodeServer = ShopStyleNodeServer;
